/*
  Copyright Jesús Pérez <jesusprubio@gmail.com>

  This code may only be used under the MIT license found at
  https://opensource.org/licenses/MIT.
*/

'use strict';

// https://developer.shodan.io/api/exploits/rest

const utils = require('./utils');

// Optional parameters for each function.
const optionalParams = {
  search: ['facets', 'page'],
  count: ['facets'],
};
// Part of all urls.
const apiSection = 'exploits';


function req(subPath, query, key, opts = {}) {
  let partialUrl = `/api/${subPath}?`;

  if (!query) { return Promise.reject(utils.createErrMandatory('query')); }

  partialUrl = utils.addParam(partialUrl, 'query', query);
  partialUrl = utils.addOptionals(partialUrl, optionalParams[subPath], opts);

  return utils.apiRequest(apiSection, partialUrl, key, opts.timeout);
}


module.exports.search = (query, key, opts = {}) => req('search', query, key, opts);


module.exports.count = (query, key, opts = {}) => req('count', query, key, opts);
